import {ModelComponent} from "../ModelComponent";
import {ItemGroup} from "./ItemGroup";
import {ItemgroupCharacteristicValue} from "./ItemgroupCharacteristicValue";
import {ItemGroupCharacteristic} from "./ItemgroupCharacteristic";

export class Item extends ModelComponent {


  manuallyInheritedGroups: ItemGroup[] = []
  hierarchyInheritedGroups: ItemGroup[] = []
  itemCharacteristicValues: ItemgroupCharacteristicValue[] = []

  initializeItemCharacteristics() {
    const inheritedGroups = this.manuallyInheritedGroups.concat(this.hierarchyInheritedGroups);
    inheritedGroups.forEach(itemGroup => {
      this.initializeItemCharacteristicsOfItemgroup(itemGroup);
    })
  }

  initializeItemCharacteristicsOfItemgroup(itemGroup: ItemGroup) {
    itemGroup.itemGroupCharacteristics.forEach(characteristic => {
      this.addCharacteristic(characteristic);
    })
  }

  private isValueInitialized(characteristic: ItemGroupCharacteristic) {
    return this.itemCharacteristicValues.find(value => value.key.characteristicName === characteristic.characteristicName) !== undefined
  }

  addInheritedGroup(itemgroup: ItemGroup) {
    if(this.findItemgroupByName(itemgroup.componentName) == undefined) {
      this.manuallyInheritedGroups.push(itemgroup);
      itemgroup.manuallyInheritedItems.push(this);
      this.initializeItemCharacteristicsOfItemgroup(itemgroup);
    }
  }

  findItemgroupByName(groupName: string) {
    const itemgroups = this.hierarchyInheritedGroups.concat(this.manuallyInheritedGroups);
    return itemgroups.find(group => group.componentName === groupName);
  }

  addCharacteristic(characteristic: ItemGroupCharacteristic) {
    const characteristicValue = new ItemgroupCharacteristicValue(characteristic, 0);
    if(!this.isValueInitialized(characteristic)) {
      this.itemCharacteristicValues.push(characteristicValue);
    }
  }

  deleteInheritedGroup(itemgroup: ItemGroup) {
    this.manuallyInheritedGroups = this.manuallyInheritedGroups.filter(manually => manually.componentName !== itemgroup.componentName);
  }

  addInheritedHierarchyGroup(itemgroup: ItemGroup) {
    this.hierarchyInheritedGroups.push(itemgroup);
    this.initializeItemCharacteristicsOfItemgroup(itemgroup);

  }
}
